@@ -1,7 +1,7 @@
 
 //  RcppRedis -- Rcpp bindings to Hiredis for some Redis functionality
 //
-//  Copyright (C) 2013 - 2024    Dirk Eddelbuettel
+//  Copyright (C) 2013 - 2025    Dirk Eddelbuettel
 //  Portions Copyright (C) 2013  Wush Wu
 //  Portions Copyright (C) 2013  William Pleasant
 //  Portions Copyright (C) 2015  Russell S. Pierce
@@ -36,7 +36,7 @@
 //
 // We do use 'binary' serialization for faster processing
 //
-// Dirk Eddelbuettel, 2013 - 2023
+// Dirk Eddelbuettel, 2013 - 2025
 
 // [[Rcpp::depends(BH)]]
 
@@ -101,8 +101,7 @@ private:
     }
 
     void *redisCommandArgvNULLSafe(redisContext *c, int argc, const char **argv, const size_t *argvlen) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandArgv(c, argc, argv, argvlen));
+        redisReply *reply = static_cast<redisReply*>(redisCommandArgv(c, argc, argv, argvlen));
         nullReplyCheck(reply);
         return reply;
     }
@@ -187,6 +186,9 @@ public:
     Redis(std::string host)                                           { init(host);             }
     Redis()                                                           { init();                 }
 
+    // used in functions below
+    static const unsigned int szdb = sizeof(double);
+
     // could create new functions to (re-)connect with given host and port etc pp
 
     ~Redis() {
@@ -196,8 +198,7 @@ public:
 
     // execute given string
     SEXP exec(std::string cmd) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, cmd.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, cmd.c_str()));
         SEXP rep = extract_reply(reply);
         freeReplyObject(reply);
         return(rep);
@@ -207,13 +208,11 @@ public:
         std::vector<const char*> cmdv(cmd.size());
         std::vector<size_t> cmdlen(cmd.size());
         for (unsigned int i=0; i < cmd.size(); ++i) {
-          cmdv[i] = cmd[i].c_str();
-          cmdlen[i] = cmd[i].size();
+            cmdv[i] = cmd[i].c_str();
+            cmdlen[i] = cmd[i].size();
         }
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandArgvNULLSafe(prc_, cmd.size(),
-                                                              &(cmdv[0]), &(cmdlen[0])));
+        redisReply *reply = static_cast<redisReply*>(redisCommandArgvNULLSafe(prc_, cmd.size(), &(cmdv[0]), &(cmdlen[0])));
         SEXP rep = extract_reply(reply);
         freeReplyObject(reply);
         return(rep);
@@ -244,6 +243,18 @@ public:
         return(exec("EXPIRE " + key + " " + ms));
     }
 
+    // redis DEL key1 [key2 key3 ...]
+    SEXP del(std::vector<std::string> sv) {
+        std::string cmd = "DEL";
+        for (auto s: sv) {
+            cmd += " " + s;
+        }
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, cmd.c_str()));
+        SEXP rep = extract_reply(reply);
+        freeReplyObject(reply);
+        return(rep);
+    }
+
     // redis set -- serializes to R internal format
     std::string set(std::string key, SEXP s) {
 
@@ -251,9 +262,7 @@ public:
         Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : R::serializeToRaw(s);
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SET %s %b",
-                                                          key.c_str(), x.begin(), x.size()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SET %s %b", key.c_str(), x.begin(), x.size()));
         std::string res(reply->str);
         freeReplyObject(reply);
         return(res);
@@ -262,8 +271,7 @@ public:
     // redis get -- deserializes from R format
     SEXP get(std::string key) {
         SEXP obj;
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "GET %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "GET %s", key.c_str()));
 
         if (replyTypeToInteger(reply) == replyNil_t) {
             obj = R_NilValue;
@@ -284,9 +292,7 @@ public:
         Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : R::serializeToRaw(s);
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HSET %s %s %b",
-                                                  key.c_str(), field.c_str(), x.begin(), x.size()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HSET %s %s %b", key.c_str(), field.c_str(), x.begin(), x.size()));
 
         checkReplyType(reply, replyInteger_t); // ensure we got integer
         int res = reply->integer;
@@ -297,9 +303,7 @@ public:
     // redis hget -- deserializes from R format
     SEXP hget(std::string key, std::string field) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HGET %s %s",
-                                                          key.c_str(), field.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HGET %s %s", key.c_str(), field.c_str()));
 
         int nc = reply->len;
         Rcpp::RawVector res(nc);
@@ -312,9 +316,7 @@ public:
     // redis hexists -- returns int
     int hexists(std::string key, std::string field) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HEXISTS %s %s",
-                                                          key.c_str(), field.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HEXISTS %s %s", key.c_str(), field.c_str()));
 
         checkReplyType(reply, replyInteger_t); // ensure we got integer
         int res = reply->integer;
@@ -325,9 +327,7 @@ public:
     // redis hdel -- returns int
     int hdel(std::string key, std::string field) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HDEL %s %s",
-                                                          key.c_str(), field.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HDEL %s %s", key.c_str(), field.c_str()));
 
         checkReplyType(reply, replyInteger_t); // ensure we got integer
         int res = reply->integer;
@@ -338,8 +338,7 @@ public:
     // redis hlen -- returns int
     int hlen(std::string key) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HLEN %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HLEN %s", key.c_str()));
 
         checkReplyType(reply, replyInteger_t); // ensure we got integer
         int res = reply->integer;
@@ -350,8 +349,7 @@ public:
     // redis hkeys -- returns character vector
     SEXP hkeys(std::string key) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HKEYS %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HKEYS %s", key.c_str()));
 
         unsigned int nc = reply->elements;
         Rcpp::CharacterVector vec(nc);
@@ -365,8 +363,7 @@ public:
     // redis hgetall -- returns a list
     Rcpp::List hgetall(std::string key) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HGETALL %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "HGETALL %s", key.c_str()));
 
         unsigned int nc = reply->elements;
         Rcpp::List vec(nc/2);
@@ -396,8 +393,7 @@ public:
         size_t cmdlen[3] = {4, key.length(), static_cast<size_t>(x.size())};
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandArgvNULLSafe(prc_, 3, cmdv, cmdlen));
+        redisReply *reply = static_cast<redisReply*>(redisCommandArgvNULLSafe(prc_, 3, cmdv, cmdlen));
         SEXP rep = extract_reply(reply);
         freeReplyObject(reply);
         return(rep);
@@ -412,8 +408,7 @@ public:
         size_t cmdlen[3] = {4, key.length(), static_cast<size_t>(x.size())};
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandArgvNULLSafe(prc_, 3, cmdv, cmdlen));
+        redisReply *reply = static_cast<redisReply*>(redisCommandArgvNULLSafe(prc_, 3, cmdv, cmdlen));
         SEXP rep = extract_reply(reply);
         freeReplyObject(reply);
         return(rep);
@@ -422,8 +417,7 @@ public:
     // redis smembers -- deserializes from R format
     Rcpp::List smembers(std::string key) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SMEMBERS %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SMEMBERS %s", key.c_str()));
 
         unsigned int len = reply->elements;
         Rcpp::List x(len);
@@ -442,8 +436,7 @@ public:
     // redis keys -- returns character vector
     SEXP keys(std::string regexp) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "KEYS %s", regexp.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "KEYS %s", regexp.c_str()));
 
         unsigned int nc = reply->elements;
         Rcpp::CharacterVector vec(nc);
@@ -455,21 +448,54 @@ public:
     }
 
     // redis ltrim: trim list so that it contains only the range of elements specified
-     SEXP ltrim(std::string key, int start, int end) {
+    SEXP ltrim(std::string key, int start, int end) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LTRIM %s %d %d",
-                                                          key.c_str(), start, end));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LTRIM %s %d %d", key.c_str(), start, end));
         SEXP rep = extract_reply(reply);
+        freeReplyObject(reply);
         return(rep);
     }
 
+    // redis lrem: removes 'count' elements 's' under 'key'
+    // count has overloaded semantics:
+    //   -N removes the last N matching s
+    //    N removes the first N matching s
+    //    0 removes all matching s
+    SEXP lrem(std::string key, int count, SEXP s) {
+        // if raw, use as is else serialize to raw
+        Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : R::serializeToRaw(s);
+
+        // uses binary protocol, see hiredis doc at github
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LREM %s %d %b", key.c_str(), count, x.begin(), x.size()));
+        SEXP rep = extract_reply(reply);
+        freeReplyObject(reply);
+        return(rep);
+    }
+
+    // redis lmove: return first/last from src and places it first/last at dest
+    // this uses R serialization
+    SEXP lmove(std::string src, std::string dest, std::string wherefrom, std::string whereto) {
+        SEXP obj;
+        std::string res;
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LMOVE %s %s %s %s", src.c_str(), dest.c_str(), wherefrom.c_str(), whereto.c_str()));
+        if (replyTypeToInteger(reply) == replyNil_t) {
+            obj = R_NilValue;
+        } else {
+            checkReplyType(reply, replyString_t); // ensure we got string
+            int nc = reply->len;
+            Rcpp::RawVector res(nc);
+            memcpy(res.begin(), reply->str, nc);
+            obj = R::unserializeFromRaw(res);
+        }
+        freeReplyObject(reply);
+        return(obj);
+    }
+
+
     // redis lrange: get list from start to end -- with R serialization
     Rcpp::List lrange(std::string key, int start, int end) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d",
-                                                          key.c_str(), start, end));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d", key.c_str(), start, end));
 
         unsigned int len = reply->elements;
         //Rcpp::Rcout << "Seeing " << len << " elements\n";
@@ -489,8 +515,7 @@ public:
 
     // redis llen: get list length
     double llen(std::string key) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LLEN %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LLEN %s", key.c_str()));
         checkReplyType(reply, replyInteger_t); // ensure we got int
         double res = static_cast<double>(reply->integer); // a 'long long' would overflow int
         freeReplyObject(reply);
@@ -503,8 +528,7 @@ public:
         SEXP obj;
         std::string res;
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPOP %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPOP %s", key.c_str()));
 
         if (replyTypeToInteger(reply) == replyNil_t) {
             obj = R_NilValue;
@@ -515,16 +539,15 @@ public:
             memcpy(res.begin(), reply->str, nc);
             obj = R::unserializeFromRaw(res);
         }
-
+        freeReplyObject(reply);
         return(obj);
     }
 
-  // redis "rpop from" -- with R serialization
+    // redis "rpop from" -- with R serialization
     SEXP rpop(std::string key) {
         SEXP obj;
         std::string res;
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "RPOP %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "RPOP %s", key.c_str()));
 
         if (replyTypeToInteger(reply) == replyNil_t) {
             obj = R_NilValue;
@@ -535,7 +558,7 @@ public:
             memcpy(res.begin(), reply->str, nc);
             obj = unserializeFromRaw(res);
         }
-
+        freeReplyObject(reply);
         return(obj);
     }
 
@@ -545,29 +568,22 @@ public:
         // if raw, use as is else serialize to raw
         Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : R::serializeToRaw(s);
 
-      // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPUSH %s %b",
-                                                          key.c_str(), x.begin(),
-                                                          x.size()*szdb));
-
+        // uses binary protocol, see hiredis doc at github
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPUSH %s %b", key.c_str(), x.begin(), x.size()));
 
         SEXP rep = extract_reply(reply);
         freeReplyObject(reply);
         return(rep);
     }
 
-      // redis "append to list" -- with R serialization
+    // redis "append to list" -- with R serialization
     SEXP rpush(std::string key, SEXP s) {
 
         // if raw, use as is else serialize to raw
         Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : R::serializeToRaw(s);
 
-      // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "RPUSH %s %b",
-                                                          key.c_str(), x.begin(),
-                                                          x.size()*szdb));
+        // uses binary protocol, see hiredis doc at github
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "RPUSH %s %b", key.c_str(), x.begin(), x.size()));
 
         SEXP rep = extract_reply(reply);
         freeReplyObject(reply);
@@ -577,14 +593,9 @@ public:
 
     // Some "non-R-serialzation" functions below
 
-    // used in functions below
-    static const unsigned int szdb = sizeof(double);
-
     // redis set a string without deserializes from R format [as set() above does]
     std::string setString(std::string key, std::string value) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SET %s %s",
-                                                          key.c_str(), value.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SET %s %s", key.c_str(), value.c_str()));
         std::string res(reply->str);
         freeReplyObject(reply);
         return(res);
@@ -592,8 +603,7 @@ public:
 
     // redis get a string without deserializes from R format [as get() above does]
     std::string getString(std::string key) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "GET %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "GET %s", key.c_str()));
         std::string res(reply->str);
         freeReplyObject(reply);
         return(res);
@@ -603,10 +613,7 @@ public:
     // redis "set a vector" -- without R serialization, without attributes, ...
     std::string setVector(std::string key, Rcpp::NumericVector x) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SET %s %b",
-                                                          key.c_str(), x.begin(),
-                                                          x.size()*szdb));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "SET %s %b", key.c_str(), x.begin(), x.size()*szdb));
         std::string res(reply->str);
         freeReplyObject(reply);
         return(res);
@@ -631,9 +638,7 @@ public:
     // assume numerical vectors, doesn't test all that well
     Rcpp::List listRange(std::string key, int start, int end) {
 
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d",
-                                                          key.c_str(), start, end));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d", key.c_str(), start, end));
 
         //Rcpp::Rcout << "listRange got type: " << replyTypeToString(reply) << "\n";
         checkReplyType(reply, replyArray_t); // ensure we got array
@@ -665,8 +670,7 @@ public:
 
     // redis "lpop from" -- without R serialization
     std::string listLPop(std::string key) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPOP %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPOP %s", key.c_str()));
 
         std::string res;
         if (replyTypeToInteger(reply) == replyNil_t) {
@@ -684,10 +688,7 @@ public:
     std::string listLPush(std::string key, Rcpp::NumericVector x) {
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPUSH %s %b",
-                                                          key.c_str(), x.begin(),
-                                                          x.size()*szdb));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LPUSH %s %b", key.c_str(), x.begin(), x.size()*szdb));
 
         //std::string res(reply->str);
         std::string res = "";
@@ -700,10 +701,7 @@ public:
     std::string listRPush(std::string key, Rcpp::NumericVector x) {
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "RPUSH %s %b",
-                                                          key.c_str(), x.begin(),
-                                                          x.size()*szdb));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "RPUSH %s %b", key.c_str(), x.begin(), x.size()*szdb));
 
         //std::string res(reply->str);
         std::string res = "";
@@ -720,11 +718,7 @@ public:
         for (int i=0; i<x.nrow(); i++) {
             Rcpp::NumericVector y = x.row(i);
             // uses binary protocol, see hiredis doc at github
-            redisReply *reply =
-                static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZADD %s %f %b",
-                                                              key.c_str(),
-                                                              y[0],
-                                                              y.begin(), y.size()*szdb));
+            redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZADD %s %f %b", key.c_str(), y[0], y.begin(), y.size()*szdb));
             checkReplyType(reply, replyInteger_t); // ensure we got array
             res += static_cast<double>(reply->integer); // a 'long long' would overflow int
             freeReplyObject(reply);
@@ -737,9 +731,7 @@ public:
     Rcpp::NumericMatrix zrange(std::string key, int min, int max) {
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZRANGE %s %d %d",
-                                                  key.c_str(), min, max));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZRANGE %s %d %d", key.c_str(), min, max));
         checkReplyType(reply, replyArray_t); // ensure we got array
         unsigned int rows = reply->elements;
         unsigned int cols = reply->element[0]->len; // assumes all row have same nb of cols
@@ -758,10 +750,7 @@ public:
     Rcpp::NumericMatrix zrangebyscore(std::string key, double min, double max) {
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_,
-                                                          "ZRANGEBYSCORE %s %f %f",
-                                                          key.c_str(), min, max));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZRANGEBYSCORE %s %f %f", key.c_str(), min, max));
         checkReplyType(reply, replyArray_t); // ensure we got array
         unsigned int rows = reply->elements;
         unsigned int cols = reply->element[0]->len; // assumes all row have same nb of cols
@@ -785,10 +774,7 @@ public:
             std::string key(keyvec[i]);
 
             // uses binary protocol, see hiredis doc at github
-            redisReply *reply =
-                static_cast<redisReply*>(redisCommandNULLSafe(prc_,
-                                                              "ZREMRANGEBYSCORE %s %f %f",
-                                                              key.c_str(), min, max));
+            redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZREMRANGEBYSCORE %s %f %f", key.c_str(), min, max));
             checkReplyType(reply, replyInteger_t); // ensure we got array
             res[i] = static_cast<double>(reply->integer); // a 'long long' would overflow int
             freeReplyObject(reply);
@@ -800,9 +786,7 @@ public:
     double zcard(std::string key) {
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_,
-                                                          "ZCARD %s", key.c_str()));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZCARD %s", key.c_str()));
         checkReplyType(reply, replyInteger_t); // ensure we got integer
         double res = static_cast<double>(reply->integer); // a 'long long' would overflow int
         freeReplyObject(reply);
@@ -811,10 +795,7 @@ public:
 
     // redis "zcount" -- counter members in set with scores in given range
     double zcount(std::string key, double min, double max) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_,
-                                                          "ZCOUNT %s %f %f",
-                                                          key.c_str(), min, max));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "ZCOUNT %s %f %f", key.c_str(), min, max));
         checkReplyType(reply, replyInteger_t); // ensure we got array
         double res = static_cast<double>(reply->integer); // a 'long long' would overflow int
         freeReplyObject(reply);
@@ -826,9 +807,7 @@ public:
     Rcpp::CharacterVector listRangeAsStrings(std::string key, int start, int end) {
 
         // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d",
-                                                          key.c_str(), start, end));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d", key.c_str(), start, end));
 
         //Rcpp::Rcout << "listRange got type: " << replyTypeToString(reply) << "\n";
         checkReplyType(reply, replyArray_t); // ensure we got array
@@ -844,9 +823,7 @@ public:
 
 #ifdef HAVE_MSGPACK
     Rcpp::NumericMatrix msgPackMatrix(std::string key, int start, int end) {
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d",
-                                                          key.c_str(), start, end));
+        redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "LRANGE %s %d %d", key.c_str(), start, end));
 
         std::vector<double> row;
         std::vector<std::vector<double> > vecs;
@@ -966,10 +943,7 @@ end:
         for (int i=0; i<n; i++) {
             std::string key(channels[i]);
 
-            redisReply *reply =
-                static_cast<redisReply*>(redisCommandNULLSafe(prc_,
-                                                              "%s %s",
-                                                              type, key.c_str()));
+            redisReply *reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "%s %s", type, key.c_str()));
             vec[i] = extract_reply(reply);
             freeReplyObject(reply);
         }
@@ -977,30 +951,36 @@ end:
     }
 
     SEXP subscribe(Rcpp::CharacterVector channels) {
-
       return(subscribe_proto(channels, "SUBSCRIBE"));
     }
 
     SEXP psubscribe(Rcpp::CharacterVector channels) {
-
       return(subscribe_proto(channels, "PSUBSCRIBE"));
     }
 
     SEXP unsubscribe(Rcpp::CharacterVector channels) {
-
       return(subscribe_proto(channels, "UNSUBSCRIBE"));
     }
 
     // redis publish -- serializes to R internal format
-    SEXP publish(std::string channel, SEXP s) {
+    SEXP publish(std::string channel, SEXP s, std::string type = "rds") {
+        redisReply *reply = nullptr;
 
-        // if raw, use as is else serialize to raw
-        Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : R::serializeToRaw(s);
-
-        // uses binary protocol, see hiredis doc at github
-        redisReply *reply =
-            static_cast<redisReply*>(redisCommandNULLSafe(prc_, "PUBLISH %s %b",
-                                                          channel.c_str(), x.begin(), x.size()));
+        if (type == "string") {
+            std::string msg = Rcpp::as<std::string>(s);
+            reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "PUBLISH %s %s", channel.c_str(), msg.c_str()));
+        } else if (type == "rds") {
+            Rcpp::RawVector x = R::serializeToRaw(s);
+            // uses binary protocol, see hiredis doc at github
+            reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "PUBLISH %s %b", channel.c_str(), x.begin(), x.size()));
+        } else if (type == "raw") {
+            Rcpp::RawVector x(s);
+            // uses binary protocol, see hiredis doc at github
+            reply = static_cast<redisReply*>(redisCommandNULLSafe(prc_, "PUBLISH %s %b", channel.c_str(), x.begin(), x.size()));
+        } else {
+            Rcpp::warning("Ignoring unsupported 'publish' type '%s'", type);
+            return(R_NilValue);
+        }
         SEXP rep = extract_reply(reply);
         freeReplyObject(reply);
         return(rep);
@@ -1024,6 +1004,7 @@ RCPP_MODULE(Redis) {
         .method("exists", &Redis::exists,  "runs 'EXISTS' command to count the number of specified keys present")
         .method("expire", &Redis::expire,  "runs 'EXPIRE' command to expire the key after a given number of seconds")
         .method("pexpire", &Redis::pexpire,  "runs 'PEXPIRE' command to expire the key after a given number of milliseconds")
+        .method("del", &Redis::del,  "runs 'DEL key1 [key2 key3 ...]' removing a key")
         .method("set",  &Redis::set,   "runs 'SET key object', serializes internally")
         .method("get",  &Redis::get,   "runs 'GET key', deserializes internally")
 
@@ -1049,6 +1030,8 @@ RCPP_MODULE(Redis) {
         .method("lrange",   &Redis::lrange,   "runs 'LRANGE key start end' for list")
         .method("llen",     &Redis::llen,     "runs 'LLEN key' for list")
         .method("ltrim",    &Redis::ltrim,    "runs 'LTRIM key start end' for list")
+        .method("lrem",     &Redis::lrem,     "runs 'LREM key count element' for list")
+        .method("lmove",    &Redis::lmove,    "runs 'LMOVE src dest <LEFT | RIGHT> <LEFT | RIGHT>' for list")
 
         // non-R serialization methods below
         .method("setString",  &Redis::setString,   "runs 'SET key obj' without serialization")
@@ -1061,7 +1044,6 @@ RCPP_MODULE(Redis) {
         .method("listRPush",  &Redis::listRPush,   "appends numeric vector to list")
         .method("listRange",  &Redis::listRange,   "runs 'LRANGE key start end' for list, native")
         .method("zadd",       &Redis::zadd,        "inserts rows of matrix into sorted set, first value is score, binary")
-
         .method("zrange",     &Redis::zrange,      "retrieve sorted range over index [min, max], binary")
         .method("zrangebyscore", &Redis::zrangebyscore,  "retrieve sorted range over score [min, max], binary")
         .method("zremrangebyscore", &Redis::zremrangebyscore,  "remove sorted range in [min, max]")
